__version__ = '0.6.0'

__uri__ = 'https://github.com/handshake/pg2kinesis'
__title__ = 'pg2kinesis'
__description__ = 'A utility that enables PostgreSQL 9.4+ to logically replicate to Amazon''s Kinesis'
__doc__ = __description__ + ' <' + __uri__ + '>'

__author__ = 'Shon T. Urbas, Geoff Johnson'
__email__ = 'shon.urbas@gmail.com, geoff.johnson@handshake.com'

__license__ = 'MIT'
__copyright__ = 'Copyright (c) 2016 Handshake Corp.'
